{% extends "admin/change_list.html" %}
{% load i18n admin_urls %}

{% block object-tools-items %}
    <li>
        <a href="{% url 'admin:chat-compose' %}" class="addlink">
            {% translate "Compose Message" %}
        </a>
    </li>
{% endblock %} 